#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "player.h"
#include "files.h"
#include "main.h"

// This function reads the lines of text from
// a file. It returns a pointer to a random line
// of text. On error, a NULL is returned.
char *Random_Line(char *filename)
{
    FILE *my_file;
    char line[MAX_INSULT_LINE];
    char *return_value;
    char *result;
    int line_count = 0;
    int grab_line;

    if (! filename) return NULL;
    my_file = fopen(filename, "r");
    if (! my_file) return NULL;

    // count the lines inthe file
    result = fgets(line, MAX_INSULT_LINE, my_file);
    while (result)
    {
       line_count++;
       result = fgets(line, MAX_INSULT_LINE, my_file);
    }

    if (! line_count) return NULL;

    // pick a random value
    fseek(my_file, 0, SEEK_SET);
    grab_line = rand() % line_count;

    // find the selected line
    line_count = 0;
    while (line_count < grab_line)
    {
        fgets(line, MAX_INSULT_LINE, my_file);
        line_count++;
    }
    fclose(my_file);

    // remove CR and LF
    if ( strchr(line, '\n') )
       strchr(line, '\n')[0] = '\0';
    if ( strchr(line, '\r') )
       strchr(line, '\r')[0] = '\0';
    // return the line we read
    return_value = (char *) calloc( strlen(line) + 1, sizeof(char) );
    if (! return_value) return NULL;
    strcpy(return_value, line);
    return return_value;    
}



/*
This function saves the game in progress.
All data is saved in a text file for flexiblity.
Returns TRUE on success and FALSE on failure.
*/
int Save_Game(GLOBALDATA *global, ENVIRONMENT *env)
{
   FILE *game_file = NULL;
   char *file_path = NULL;
   int player_count = 0;
   PLAYER *my_player = NULL;
   int count;
   char *homedir;
   int global_player_number;

    // figure out file name
   homedir = getenv(HOME_DIR);
   if (! homedir)
     homedir = ".";
   file_path = (char *) calloc( strlen(homedir) +
                                strlen(global->game_name) + 24, 
                                sizeof(char) );
   if (! file_path)
      return FALSE;

   sprintf(file_path, "%s/.atanks/%s.sav", homedir, global->game_name);

   game_file = fopen(file_path, "w");
   free(file_path);
   if (!game_file)
      return FALSE;

   // write global data
   fprintf(game_file, "GLOBAL\n");
   fprintf(game_file, "ROUNDS=%lf\n", global->rounds);
   fprintf(game_file, "CURRENTROUND=%d\n", global->currentround);
   fprintf(game_file, "***\n");

   // write envrionment data
   fprintf(game_file, "ENVIRONMENT\n");
   fprintf(game_file, "***\n");

   // write player data
   fprintf(game_file, "PLAYERS\n");
   while ( player_count < global->numPlayers )
   {
       my_player = global->players[player_count];
       global_player_number = 0;
       while ( strcmp(global->allPlayers[global_player_number]->getName(), my_player->getName() ) )
           global_player_number++;
       fprintf(game_file, "PLAYERNUMBER=%d\n", global_player_number);
       fprintf(game_file, "SCORE=%d\n", my_player->score);
       fprintf(game_file, "MONEY=%d\n", my_player->money);
       for (count = 0; count < WEAPONS; count++)
           fprintf(game_file, "WEAPON=%d %d\n", count, my_player->nm[count]);
       for (count = 0; count < ITEMS; count++)
           fprintf(game_file, "ITEM=%d %d\n", count, my_player->ni[count]);
       fprintf(game_file, "***\n");    
       player_count++;
   }

   fclose(game_file);

   // save the current config
   file_path = (char *) calloc( strlen(homedir) +
                                strlen(global->game_name) + 24,
                                sizeof(char) );
   if (! file_path)
       return FALSE;
   sprintf(file_path, "%s/.atanks/%s.txt", homedir, global->game_name);
   Save_Game_Settings_Text(global, env, file_path);
   free(file_path);

   return TRUE;
}



/*
This function attempts to load a saved
game.
The function returns TRUE on success and
FALSE is an error occures.
-- Jesse
*/
int Load_Game(GLOBALDATA *global, ENVIRONMENT *env)
{
   char line[512];
   char *field, *value;
   int index, amount, player_count = 0;
   FILE *my_file;
   char *file_path;
   char *homedir;
   int stage = NO_STAGE;
   char *got_line;
   int global_player_number;

   // first things first, get the file name
   homedir = getenv(HOME_DIR);
   if (! homedir)
       homedir = ".";

   // load config settings
   /*
   file_path = (char *) calloc( strlen(homedir) +
                                strlen(global->game_name) + 24,
                                sizeof(char) );
   if (! file_path)
     return FALSE;
   sprintf(file_path, "%s/.atanks/%s.txt", homedir, global->game_name);
   my_file = fopen(file_path, "r");
   free(file_path);
   if (! my_file)
     return FALSE;
   global->loadFromFile_Text(my_file);
   env->loadFromFile_Text(my_file);
   fclose(my_file);
   */

   file_path = (char *) calloc( strlen(homedir) +
                                strlen(global->game_name) + 24,
                                sizeof(char) );
   if (! file_path)
      return FALSE;

   sprintf(file_path, "%s/.atanks/%s.sav", homedir, global->game_name);

   my_file = fopen(file_path, "r");
   free(file_path);
   if (! my_file)
      return FALSE;

   // read a line from the file
   got_line = fgets(line, 512, my_file);
   // keep reading until we hit EOF or error
   while ( (got_line) && (player_count < MAXPLAYERS) )
   {
        // clear end of line
        if ( strchr(line, '\n') )
             strchr(line, '\n')[0] = '\0';
        if ( strchr(line, '\r') )
             strchr(line, '\r')[0] = '\0';

        // check to see if we found a new stage
        if (! strcasecmp(line, "global") )
           stage = GLOBAL_STAGE;
        else if (! strcasecmp(line, "environment") )
           stage = ENVIRONMENT_STAGE;
        else if (! strcasecmp(line, "players") )
           stage = PLAYER_STAGE;
       
        // not a new stage, seperate field and value
        else
        {
           value = strchr(line, '=');
           if (value)        // valid line
           {
              field = line;
              value[0] = '\0';   // seperate field and value;
              value++;           // go to first place after =
              // interpret data
              switch (stage)
              {
                 case GLOBAL_STAGE:
                 if (! strcasecmp(field, "rounds") )
                    sscanf(value, "%lf", &global->rounds);
                 else if (! strcasecmp(field, "currentround") )
                    sscanf(value, "%d", &global->currentround);
   
                 break;
                 case ENVIRONMENT_STAGE:

                 break;
                 case PLAYER_STAGE:
                     if (! strcasecmp(field, "playernumber") )
                     {
                        sscanf(value, "%d", &global_player_number);
                        global->addPlayer( global->allPlayers[global_player_number] );
                     }
                     else if (! strcasecmp(field, "score") )
                        sscanf(value, "%d", &(global->players[player_count]->score) );
                     else if (! strcasecmp(field, "money") )
                        sscanf(value, "%d", &(global->players[player_count]->money) );
                     else if (! strcasecmp(field, "weapon") )
                     {
                        sscanf(value, "%d %d", &index, &amount);
                        global->players[player_count]->nm[index] = amount;
                     }
                     else if (! strcasecmp(field, "item") )
                     {
                        sscanf(value, "%d %d", &index, &amount);
                        global->players[player_count]->ni[index] = amount;
                     }
                 break;
               }    // end of stage

           }       //     end of valid line
           else if ( (! strcmp(line, "***") ) && (stage == PLAYER_STAGE) )
               player_count++;

        }     // end of field value area

        // read next line of file
        got_line = fgets(line, 512, my_file);

   }    // end of reading file
   fclose(my_file);
   return TRUE;
}



/*
Check to see if a saved game exists with the given name.
*/
int Check_For_Saved_Game(GLOBALDATA *global)
{
    FILE *my_file;
    char *path_name;
    char *homeDir;
    
    homeDir = getenv(HOME_DIR);
    if (! homeDir)
       homeDir = ".";

    path_name = (char *) calloc( strlen(homeDir) + strlen(global->game_name) + 24,
                                 sizeof(char) );
    if (! path_name)
       return FALSE;

    sprintf(path_name, "%s/.atanks/%s.sav", homeDir, global->game_name);

    my_file = fopen(path_name, "r");
    free(path_name);
    if (my_file)
    {
       fclose(my_file);
       return TRUE;
    }
    else
       return FALSE;

}



/*
This function copies the atanks config file
from the HOME_DIR folder to HOME_DIR/.atanks
If the .stanks folder does not exist, this
function will create it.
Returns TRUE on success and FALSE on error.
*/
int Copy_Config_File()
{
   FILE *source_file, *dest_file;
   char source_path[1024], dest_path[1024];
   char buffer[1024];
   char *my_home_folder;
   int status;

   // figure out where home is
   my_home_folder = getenv(HOME_DIR);
   if (! my_home_folder)
      my_home_folder = ".";

   // check to see if the config file has already been copied
   snprintf(source_path, 1024, "%s/.atanks/atanks-config.txt", my_home_folder);
   source_file = fopen(source_path, "r");
   if (source_file)     // config file is n the right place
   {
      fclose(source_file);
      return TRUE;
   }

   /*
   // check to make sure we have a source file
   snprintf(source_path, 1024, "%s/.atanks-config.txt", my_home_folder);
   source_file = fopen(source_path, "r");
   if (! source_file)
      return TRUE;
   */

   // file not copied yet, create the required directory
   snprintf(buffer, 1024, "%s/.atanks", my_home_folder);
   status = mkdir(buffer, 0700);
   if (status == -1)
   {
       printf("Error occured. Unable to create sub directory.\n");
       fclose(source_file);
       return FALSE;
   }

   // check to make sure we have a source file
   snprintf(source_path, 1024, "%s/.atanks-config.txt", my_home_folder);
   source_file = fopen(source_path, "r");
   if (! source_file)
      return TRUE;

   // we already have an open source file, create destination file
   snprintf(dest_path, 1024, "%s/.atanks/atanks-config.txt", my_home_folder);
   dest_file = fopen(dest_path, "wb");
   if (! dest_file)
   {
        printf("Unable to create destination file.\n");
        fclose(source_file);
        return FALSE;
   }

   // we have open files, let's copy
   status = fread(buffer, 1, 1024, source_file);
   while (status)
   {
       fwrite(buffer, 1, 1024, dest_file);
       status = fread(buffer, 1, 1024, source_file);
   }
   
   fclose(source_file);
   fclose(dest_file);
   return TRUE;
}



/*
Displays lines of text.
*/ 
void renderTextLines (GLOBALDATA *global, ENVIRONMENT *env,
                const LINESEQ& lines, int scrollOffset,
                const FONT* fnt, const int spacing )
{
        const int textheight = text_height (fnt) ;
        const int textheight_s = textheight * spacing ;
        int yOffset = scrollOffset;

        for (int count = 0;
                (count != lines.size()) && (yOffset < global->screenHeight);
                count++)
        {
                const char *text = lines[count] + 1;
                const char header = lines[count][0];
                int tLen = text_length (fnt, text);

                switch (header) {
                case 'T':
                case 't':
                        textout_centre_ex (env->db, fnt, text,
                                global->halfWidth + 2, global->halfHeight + yOffset + 2, BLACK, -1);
                        textout_centre_ex (env->db, fnt, text,
                                global->halfWidth, global->halfHeight + yOffset, WHITE, -1);
                        if(header == 'T') {
                                yOffset += textheight + 1;
                                hline (env->db, global->halfWidth - tLen / 2,
                                        global->halfHeight + yOffset, global->halfWidth + tLen / 2,
                                        WHITE);
                                hline (env->db, global->halfWidth - tLen / 2 + 2,
                                        global->halfHeight + yOffset + 2,
                                        global->halfWidth + tLen / 2 + 2, BLACK);
                                yOffset -= textheight / 2;
                        }
                        break;
                case 'L':
                        textout_centre_ex (env->db, fnt, text, global->halfWidth,
                                global->halfHeight + yOffset, BLACK, -1);
                        break;
                }
                yOffset += textheight_s;
        }
}




/*
Scroll text in a box
*/
void scrollTextList (GLOBALDATA *global, ENVIRONMENT *env,
        const LINESEQ& lines)
{
        /* Justin says: this function, along with renderTextLines, aren't
        exactly efficient.  I think they could use rewrite. */
        static const int numItemsSrc[] = { 100, 30 } ;

        DATAFILE *dffont ;
        const FONT* fnt = font;
        int spacing = 2;
        int tOffset = rand ();
        int itemType = rand () / (RAND_MAX/2 + 1) ;
        int numItems = numItemsSrc[itemType];

        /* Attempt to load the "big" font */
        if(( dffont = load_datafile_object("bigfont.dat", "bigfont") )) {
                fnt = (FONT*)dffont->dat ;
                spacing = 1 ;
        }

        draw_circlesBG (global, env->db, 0, 0, global->screenWidth, global->screenHeight);
        drawMenuBackground (global, env, BACKGROUND_BLANK, abs (tOffset), numItems);
        quickChange (global, env->db);
        set_clip_rect (env->db, global->halfWidth - 300 + 1, 100 + 1,
                global->halfWidth + 300 - 1, global->screenHeight - 100 - 1);
        int scrollOffset = 0;
        flush_inputs ();
        if (! global->os_mouse) show_mouse (NULL);

        do {
                drawMenuBackground (global, env, BACKGROUND_BLANK, abs (tOffset), numItems);
                renderTextLines (global, env, lines, scrollOffset,
                        fnt, spacing );
                blit (env->db, screen, global->halfWidth - 300, 100, global->halfWidth - 300, 100,
                        601, global->screenHeight - 199);
                LINUX_REST;
                tOffset++;
                scrollOffset--;
                if (scrollOffset < -(global->halfHeight - 100 + lines.size() * 20))
                        scrollOffset = global->halfHeight - 100;
        } while (!keypressed () && !mouse_b);

        if (! global->os_mouse) show_mouse (screen);
        set_clip_rect (env->db, 0, 0, (global->screenWidth-1), (global->screenHeight-1));
        flush_inputs ();
        if( dffont )
                unload_datafile_object( dffont );
}



/* Flush key buffer and waits for button releases */
void flush_inputs()  
{
        do { } while(mouse_b);
        clear_keybuf();
}

